@@ -16,6 +16,7 @@
     Foundation, 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
 
+#include <algorithm>
 #include <cctype>
 #include <cstdio>
 #include <stack>
@@ -81,205 +82,6 @@ bool pbm_getbit( FILE * f ) throw( Bitmap::Error )
 } // end namespace
 
 
-Bitmap::Bitmap( FILE * f, bool invert ) throw( Error )
-  {
-  char filetype = 0;
-
-  if( pbm_getrawbyte( f ) == 'P' )
-    {
-    char ch = pbm_getrawbyte( f );
-    if( ch == '1' || ch == '4' ) filetype = ch;
-    }
-  if( filetype == 0 )
-    throw Error( "bad magic number - not a pbm file.\n" );
-
-  _width = pbm_getint( f );
-  _height = pbm_getint( f );
-
-  data.resize( _height );
-
-  if( filetype == '1' )
-    {
-    for( int row = 0; row < _height; ++row )
-      {
-      data[row].resize( _width );
-      if( invert )
-        for( int col = 0; col < _width; ++col )
-          data[row][col] = !pbm_getbit( f );
-      else
-        for( int col = 0; col < _width; ++col )
-          data[row][col] = pbm_getbit( f );
-      }
-    }
-  else
-    {
-    for( int row = 0; row < _height; ++row )
-      {
-      data[row].resize( _width );
-      unsigned char byte = 0, mask = 0;
-      if( invert )
-        for( int col = 0; col < _width; ++col )
-          {
-          if( mask == 0 ) { mask = 0x80; byte = pbm_getrawbyte( f ); }
-          data[row][col] = !( byte & mask );
-          mask >>= 1;
-          }
-      else
-        for( int col = 0; col < _width; ++col )
-          {
-          if( mask == 0 ) { mask = 0x80; byte = pbm_getrawbyte( f ); }
-          data[row][col] = ( byte & mask );
-          mask >>= 1;
-          }
-      }
-    }
-  }
-
-
-Bitmap::Bitmap( const Bitmap & source, const Rectangle & r ) throw()
-  {
-  if( r.right() >= source._width || r.bottom() >= source._height )
-    Ocrad::internal_error( "bad parameter building a Bitmap from another one" );
-
-  _width = r.right() - r.left() + 1;
-  _height = r.bottom() - r.top() + 1;
-
-  data.resize( _height );
-  for( int row = 0; row < _height; ++row )
-    {
-    data[row].resize( _width );
-
-    for( int col = 0; col < _width; ++col )
-      data[row][col] = source.data[r.top()+row][r.left()+col];
-    }
-  }
-
-
-Bitmap::Bitmap( const Bitmap & source, int scale, int th ) throw()
-  {
-  if( scale < 2 || scale > source._width || scale > source._height ||
-      th >= scale * scale )
-    Ocrad::internal_error( "bad parameter building a reduced Bitmap" );
-
-  if( th < 0 )
-    {
-    long dots = 0;
-    for( int row = 0; row < source._height; ++row )
-      for( int col = 0; col < source._width; ++col )
-        if( source.data[row][col] ) ++dots;
-    th = ( (scale * scale * dots) - 1 ) / (source._width * source._height);
-    std::fprintf( stderr, "automatic threshold = %d\n", th );
-    }
-
-  _width = source._width / scale;
-  _height = source._height / scale;
-
-  data.resize( _height );
-  for( int row = 0; row < _height; ++row )
-    {
-    int srow = (row * scale) + scale;
-    data[row].resize( _width );
-    for( int col = 0; col < _width; ++col )
-      {
-      int scol = (col * scale) + scale;
-      int counter = 0;
-      for( int i = srow - scale; i < srow; ++i )
-        for( int j = scol - scale; j < scol; ++j )
-          if( source.data[i][j] && ++counter > th ) goto L1;
-      L1: data[row][col] = ( counter > th );
-      }
-    }
-  }
-
-
-Bitmap::Bitmap( const Bitmap & source, type t ) throw( Error )
-  {
-  _width = source._width; _height = source._height;
-
-  data.resize( _height );
-  for( int row = 0; row < _height; ++row ) data[row].resize( _width );
-
-  if( t == vertical_histogram )
-    {
-    for( int col = 0; col < _width; ++col )
-      for( int row = 0, y = _height; row < _height; ++row )
-        if( source.data[row][col] ) data[--y][col] = true;
-    }
-  else if( t == horizontal_histogram )
-    {
-    for( int row = 0; row < _height; ++row )
-      for( int col = 0, x = 0; col < _width; ++col )
-        if( source.data[row][col] ) data[row][x++] = true;
-    }
-  else if( t == connected_ground )
-    {
-    int row, col;
-    for( row = 0; row < _height; ++row )
-      {
-      for( col = 0; col < _width; ++col )
-        if( source.data[row][col] ) break;
-      if( col == _width ) break;
-      }
-    if( row == _height ) throw Error( "image is not clear enough.\n" );
-    floodfill4( source, *this, row, _width/2 );
-    }
-  else Ocrad::internal_error( "type not implemented building a 'special' Bitmap" );
-  }
-
-
-void Bitmap::floodfill4( const Bitmap & source, Bitmap & dest,
-                         int row, int col ) throw()
-  {
-  std::stack< int, std::vector< int > > st;
-
-  if( source.data[row][col] == false && dest.data[row][col] == false )
-    { dest.data[row][col] = true; st.push(row); st.push(col); }
-
-  while( !st.empty() )
-    {
-    col = st.top(); st.pop();
-    row = st.top(); st.pop();
-    if( col > 0 && !source.data[row][col-1] && !dest.data[row][col-1] )
-      { dest.data[row][col-1] = true; st.push(row); st.push(col-1); }
-    if( col < dest._width-1 && !source.data[row][col+1] && !dest.data[row][col+1] )
-      { dest.data[row][col+1] = true; st.push(row); st.push(col+1); }
-    if( row > 0 && !source.data[row-1][col] && !dest.data[row-1][col] )
-      { dest.data[row-1][col] = true; st.push(row-1); st.push(col); }
-    if( row < dest._height-1 && !source.data[row+1][col] && !dest.data[row+1][col] )
-      { dest.data[row+1][col] = true; st.push(row+1); st.push(col); }
-    }
-  }
-
-
-int Bitmap::analyse_layout( int layout_level ) throw()
-  {
-  if( _rectangle_vector.size() != 0 )
-    Ocrad::internal_error( "analyse_layout called twice" );
-
-  if( layout_level > 0 && _width > 200 && _height > 200 )
-    {
-    Bitmap reduced( *this, 10, 9 );
-    Rectangle r( 0, 0, reduced._width - 1, reduced._height - 1 );
-    reduced._rectangle_vector.push_back( r );
-    reduced.find_columns( 0 );
-    if( layout_level >= 2 ) reduced.find_rows_and_columns();
-    if( reduced.rectangles() > 1 )
-      for( int i = 0; i < reduced.rectangles(); ++i )
-        {
-        const Rectangle & r = reduced._rectangle_vector[i];
-        Rectangle r1( 10*r.left(), 10*r.top(), 10*r.right(), 10*r.bottom() );
-        _rectangle_vector.push_back( r1 );
-        }
-    }
-  if( _rectangle_vector.size() == 0 )
-    {
-    Rectangle r( 0, 0, _width - 1, _height - 1 );
-    _rectangle_vector.push_back( r );
-    }
-  return _rectangle_vector.size();
-  }
-
-
 int Bitmap::find_columns( int rindex ) throw()
   {
   const int colmin = ( _width > 100 ) ? _width / 10 : 10;
@@ -427,6 +229,261 @@ bool Bitmap::find_rows_and_columns() throw()
   }
 
 
+void Bitmap::mirror_diagonal() throw()
+  {
+  int size = std::max( _width, _height );
+
+  if( _height < size )
+    {
+    data.resize( size );
+    for( int row = _height; row < size; ++row )
+      data[row].resize( size );
+    }
+  else if( _width < size )
+    for( int row = 0; row < _height; ++row )
+      data[row].resize( size );
+
+  for( int row = 0; row < size; ++row )
+    for( int col = 0; col < row; ++col )
+      {
+      bool b = data[row][col];
+      data[row][col] = data[col][row]; data[col][row] = b;
+      }
+
+  int tmp = _width; _width = _height; _height = tmp;
+  if( _height < size ) data.resize( _height );
+  else if( _width < size )
+    for( int row = 0; row < _height; ++row )
+      data[row].resize( _width );
+  }
+
+
+void Bitmap::mirror_left_right() throw()
+  {
+  for( int row = 0; row < _height; ++row )
+    reverse( data[row].begin(), data[row].end() );
+  }
+
+
+void Bitmap::mirror_top_bottom() throw()
+  {
+  for( int u = 0, d = _height - 1; u < d; ++u, --d )
+    swap( data[u], data[d] );
+  }
+
+
+void Bitmap::floodfill4( const Bitmap & source, Bitmap & dest,
+                         int row, int col ) throw()
+  {
+  std::stack< int, std::vector< int > > st;
+
+  if( source.data[row][col] == false && dest.data[row][col] == false )
+    { dest.data[row][col] = true; st.push(row); st.push(col); }
+
+  while( !st.empty() )
+    {
+    col = st.top(); st.pop();
+    row = st.top(); st.pop();
+    if( col > 0 && !source.data[row][col-1] && !dest.data[row][col-1] )
+      { dest.data[row][col-1] = true; st.push(row); st.push(col-1); }
+    if( col < dest._width-1 && !source.data[row][col+1] && !dest.data[row][col+1] )
+      { dest.data[row][col+1] = true; st.push(row); st.push(col+1); }
+    if( row > 0 && !source.data[row-1][col] && !dest.data[row-1][col] )
+      { dest.data[row-1][col] = true; st.push(row-1); st.push(col); }
+    if( row < dest._height-1 && !source.data[row+1][col] && !dest.data[row+1][col] )
+      { dest.data[row+1][col] = true; st.push(row+1); st.push(col); }
+    }
+  }
+
+
+Bitmap::Bitmap( FILE * f, bool invert ) throw( Error )
+  {
+  char filetype = 0;
+
+  if( pbm_getrawbyte( f ) == 'P' )
+    {
+    char ch = pbm_getrawbyte( f );
+    if( ch == '1' || ch == '4' ) filetype = ch;
+    }
+  if( filetype == 0 )
+    throw Error( "bad magic number - not a pbm file.\n" );
+
+  _width = pbm_getint( f );
+  _height = pbm_getint( f );
+
+  data.resize( _height );
+
+  if( filetype == '1' )
+    {
+    for( int row = 0; row < _height; ++row )
+      {
+      data[row].resize( _width );
+      if( invert )
+        for( int col = 0; col < _width; ++col )
+          data[row][col] = !pbm_getbit( f );
+      else
+        for( int col = 0; col < _width; ++col )
+          data[row][col] = pbm_getbit( f );
+      }
+    }
+  else
+    {
+    for( int row = 0; row < _height; ++row )
+      {
+      data[row].resize( _width );
+      unsigned char byte = 0, mask = 0;
+      if( invert )
+        for( int col = 0; col < _width; ++col )
+          {
+          if( mask == 0 ) { mask = 0x80; byte = pbm_getrawbyte( f ); }
+          data[row][col] = !( byte & mask );
+          mask >>= 1;
+          }
+      else
+        for( int col = 0; col < _width; ++col )
+          {
+          if( mask == 0 ) { mask = 0x80; byte = pbm_getrawbyte( f ); }
+          data[row][col] = ( byte & mask );
+          mask >>= 1;
+          }
+      }
+    }
+  }
+
+
+Bitmap::Bitmap( const Bitmap & source, const Rectangle & r ) throw()
+  {
+  if( r.left() < 0 || r.top() < 0 ||
+      r.right() >= source._width || r.bottom() >= source._height )
+    Ocrad::internal_error( "bad parameter building a Bitmap from another one" );
+
+  _width = r.right() - r.left() + 1;
+  _height = r.bottom() - r.top() + 1;
+
+  data.resize( _height );
+  for( int row = 0; row < _height; ++row )
+    {
+    data[row].resize( _width );
+
+    for( int col = 0; col < _width; ++col )
+      data[row][col] = source.data[r.top()+row][r.left()+col];
+    }
+  }
+
+
+Bitmap::Bitmap( const Bitmap & source, int scale, int th ) throw()
+  {
+  if( scale < 2 || scale > source._width || scale > source._height ||
+      th >= scale * scale )
+    Ocrad::internal_error( "bad parameter building a reduced Bitmap" );
+
+  if( th < 0 )
+    {
+    long dots = 0;
+    for( int row = 0; row < source._height; ++row )
+      for( int col = 0; col < source._width; ++col )
+        if( source.data[row][col] ) ++dots;
+    th = ( (scale * scale * dots) - 1 ) / (source._width * source._height);
+    std::fprintf( stderr, "automatic threshold = %d\n", th );
+    }
+
+  _width = source._width / scale;
+  _height = source._height / scale;
+
+  data.resize( _height );
+  for( int row = 0; row < _height; ++row )
+    {
+    int srow = (row * scale) + scale;
+    data[row].resize( _width );
+    for( int col = 0; col < _width; ++col )
+      {
+      int scol = (col * scale) + scale;
+      int counter = 0;
+      for( int i = srow - scale; i < srow; ++i )
+        for( int j = scol - scale; j < scol; ++j )
+          if( source.data[i][j] && ++counter > th ) goto L1;
+      L1: data[row][col] = ( counter > th );
+      }
+    }
+  }
+
+
+Bitmap::Bitmap( const Bitmap & source, Type t ) throw( Error )
+  {
+  _width = source._width; _height = source._height;
+
+  data.resize( _height );
+  for( int row = 0; row < _height; ++row ) data[row].resize( _width );
+
+  if( t == vertical_histogram )
+    {
+    for( int col = 0; col < _width; ++col )
+      for( int row = 0, y = _height; row < _height; ++row )
+        if( source.data[row][col] ) data[--y][col] = true;
+    }
+  else if( t == horizontal_histogram )
+    {
+    for( int row = 0; row < _height; ++row )
+      for( int col = 0, x = 0; col < _width; ++col )
+        if( source.data[row][col] ) data[row][x++] = true;
+    }
+  else if( t == connected_ground )
+    {
+    int row, col;
+    for( row = 0; row < _height; ++row )
+      {
+      for( col = 0; col < _width; ++col )
+        if( source.data[row][col] ) break;
+      if( col == _width ) break;
+      }
+    if( row == _height ) throw Error( "image is not clear enough.\n" );
+    floodfill4( source, *this, row, _width/2 );
+    }
+  else Ocrad::internal_error( "type not implemented building a 'special' Bitmap" );
+  }
+
+
+int Bitmap::analyse_layout( int layout_level ) throw()
+  {
+  if( _rectangle_vector.size() != 0 )
+    Ocrad::internal_error( "analyse_layout called twice" );
+
+  if( layout_level > 0 && _width > 200 && _height > 200 )
+    {
+    Bitmap reduced( *this, 10, 9 );
+    Rectangle r( 0, 0, reduced._width - 1, reduced._height - 1 );
+    reduced._rectangle_vector.push_back( r );
+    reduced.find_columns( 0 );
+    if( layout_level >= 2 ) reduced.find_rows_and_columns();
+    if( reduced.rectangles() > 1 )
+      for( int i = 0; i < reduced.rectangles(); ++i )
+        {
+        const Rectangle & r = reduced._rectangle_vector[i];
+        Rectangle r1( 10*r.left(), 10*r.top(), 10*r.right(), 10*r.bottom() );
+        _rectangle_vector.push_back( r1 );
+        }
+    }
+  if( _rectangle_vector.size() == 0 )
+    {
+    Rectangle r( 0, 0, _width - 1, _height - 1 );
+    _rectangle_vector.push_back( r );
+    }
+  return _rectangle_vector.size();
+  }
+
+
+bool Bitmap::only_this_rectangle( int i ) throw()
+  {
+  if( i < 0 || i >= (int)_rectangle_vector.size() ) return false;
+  if( _rectangle_vector.size() > 1 )
+    {
+    Rectangle r = _rectangle_vector[i];
+    _rectangle_vector.clear(); _rectangle_vector.push_back( r );
+    }
+  return true;
+  }
+
+
 void Bitmap::save( FILE * f, char filetype ) const throw()
   {
   if( filetype != '1' && filetype != '4' ) filetype = '4';
@@ -456,3 +513,23 @@ void Bitmap::save( FILE * f, char filetype ) const throw()
       }
     }
   }
+
+
+void Bitmap::transform( const Transformation & t ) throw()
+  {
+  switch( t.type() )
+    {
+    case Transformation::none: break;
+    case Transformation::rotate90:  mirror_diagonal(); mirror_top_bottom();
+      break;
+    case Transformation::rotate180: mirror_left_right(); mirror_top_bottom();
+      break;
+    case Transformation::rotate270: mirror_diagonal(); mirror_left_right();
+      break;
+    case Transformation::mirror_lr: mirror_left_right(); break;
+    case Transformation::mirror_tb: mirror_top_bottom(); break;
+    case Transformation::mirror_d1: mirror_diagonal(); break;
+    case Transformation::mirror_d2:
+      mirror_diagonal(); mirror_left_right(); mirror_top_bottom(); break;
+    }
+  }
