@@ -27,6 +27,9 @@ class Bitmap
 
   int  find_columns( int rindex ) throw();
   bool find_rows_and_columns() throw();
+  void mirror_diagonal() throw();
+  void mirror_left_right() throw();
+  void mirror_top_bottom() throw();
   static void floodfill4( const Bitmap & source, Bitmap & dest,
                           int row, int col ) throw();
 public:
@@ -36,7 +39,7 @@ public:
     Error( const char * p ) { s = p; }
     };
 
-  enum type { vertical_histogram, horizontal_histogram, connected_ground };
+  enum Type { vertical_histogram, horizontal_histogram, connected_ground };
 
   // Creates a Bitmap from a pbm file
   Bitmap( FILE * f, bool invert ) throw( Error );
@@ -45,7 +48,7 @@ public:
   // Creates a reduced bitmap
   Bitmap( const Bitmap & source, int scale, int th ) throw();
   // Creates a "special" type of Bitmap
-  Bitmap( const Bitmap &, type ) throw( Error );
+  Bitmap( const Bitmap &, Type ) throw( Error );
 
   int width()  const throw() { return _width;  }
   int height() const throw() { return _height; }
@@ -54,6 +57,9 @@ public:
   const std::vector< Rectangle > & rectangle_vector() const throw()
     { return _rectangle_vector; }
   int rectangles() const throw() { return _rectangle_vector.size(); }
-  int analyse_layout( int layout_level ) throw();
+
+  int  analyse_layout( int layout_level ) throw();
+  bool only_this_rectangle( int i ) throw();
   void save( FILE *, char filetype = '4' ) const throw();
+  void transform( const Transformation & t ) throw();
   };
