@@ -437,28 +437,30 @@ void Block::shift_block( const Block & b ) throw()
 void Block::hierarchize_blocks( std::vector< Block > & block_vector ) throw()
   {
   int blocks = block_vector.size();
-  for( int i1 = 0; i1 < blocks; ++i1 )
+  for( int i1 = blocks - 2; i1 >= 0; --i1 )
     {
     for( int i2 = i1 + 1; i2 < blocks; )
       {
       Block & b1 = block_vector[i1];
       Block & b2 = block_vector[i2];
       if( b2.top() >= b1.bottom() ) break;
+      bool shift = false;
       if( b1.strictly_includes( b2 ) )
         {
         if( ( b1.id() > 0 && b2.id() < 0 ) || ( b1.id() < 0 && b2.id() > 0 ) )
-          b1.shift_block( b2 );
+          { b1.shift_block( b2 ); shift = true; }
         else
           {
           for( int i3 = 0; i3 < b1.blocks(); ++i3 )
             {
             Block & b3 = b1._block_vector[i3];
-            if( b3.strictly_includes( b2 ) ) b3.shift_block( b2 );
+            if( b3.strictly_includes( b2 ) )
+              { b3.shift_block( b2 ); shift = true; }
             }
           }
-        block_vector.erase( block_vector.begin() + i2 ); --blocks;
         }
-      else ++i2;
+      if( !shift ) ++i2;
+      else { --blocks; block_vector.erase( block_vector.begin() + i2 ); }
       }
     }
   }
