@@ -27,7 +27,7 @@
 
 namespace {
 
-void ignore_anormal_blocks( std::vector< Block > & block_vector ) throw()
+void ignore_abnormal_blocks( std::vector< Block > & block_vector ) throw()
   {
   for( int i = block_vector.size() - 1; i >= 0; --i )
     {
@@ -132,7 +132,7 @@ void Blockmap::add_point_to_block( int row, int col, int id ) throw()
       if( c < _width ) return;
       Block temp = block_vector[i];
       block_vector.erase( block_vector.begin() + i );
-      block_vector.push_back( temp );
+      block_vector.push_back( temp );	// FIXME mark this block to be ignored
       return;
       }
   Ocrad::internal_error( "add_point_to_block, lost block" );
@@ -330,7 +330,7 @@ Blockmap::Blockmap( const Bitmap & page_image, int rindex, int debug_level )
       {
       ignore_small_blocks( _block_matrix[i] );
       Block::hierarchize_blocks( _block_matrix[i] );
-      ignore_anormal_blocks( _block_matrix[i] );
+      ignore_abnormal_blocks( _block_matrix[i] );
       remove_top_bottom_noise( _block_matrix[i] );
       remove_left_right_noise( _block_matrix[i] );
       }
