@@ -163,7 +163,11 @@ void Character::recognize110( const Charset & charset, int charbox_vcenter ) thr
       add_guess( 'm', b.right() ); return; }
     }
 
-  if( f.tp.minima() == 3 ) { add_guess( 'w', 0 ); return; }
+  if( f.tp.minima() == 3 )
+    {
+    int hdiff; if( !b.bottom_hook( &hdiff ) ) add_guess( 'w', 0 );
+    return;
+    }
 
   code = f.test_line( charbox_vcenter );
   if( code ) { add_guess( code, 0 ); return; }
