@@ -91,103 +91,108 @@ void Character::recognize12( const Charset & charset, int charbox_vcenter ) thro
     }
   if( a1 < a2 )
     {
-    {
-    int code = f1.test_solid( charbox_vcenter );	//FIXME all this
-    if( code == '-' && 2 * b1.height() > b1.width() ) code = '.';
-    else if( code == '\'' || code == '|' ) code = '.';
-    if( !code && !b1.blocks() && 
-        2 * b1.height() < b2.height() && b1.width() <= b2.width() )
       {
-      if( 10 * a1 >= 7 * b1.height() * b1.width() ) code = '.';
-      else code = '\'';
-      }
-    if( !b2.blocks() && ( code == '.' || code == '\'' ) )
-      {
-      // Looks for merged 'ri' or 'r'
-      if( f2.bp.minima( b2.height() / 4 ) == 2 &&
-          b2.top() > b1.bottom() && b2.hcenter() < b1.left() )
+      int code = f1.test_solid( charbox_vcenter );	//FIXME all this
+      if( code == '-' && 2 * b1.height() > b1.width() ) code = '.';
+      else if( code == '\'' || code == '|' ) code = '.';
+      if( !code && !b1.blocks() && 
+          2 * b1.height() < b2.height() && b1.width() <= b2.width() )
         {
-        Character c2( b2 ); c2.recognize1( charset, vcenter() );
-        if( c2.maybe('n') )
+        if( 10 * a1 >= 7 * b1.height() * b1.width() ) code = '.';
+        else code = '\'';
+        }
+      if( !b2.blocks() && ( code == '.' || code == '\'' ) )
+        {
+        // Looks for merged 'ri' or 'r'
+        if( f2.bp.minima( b2.height() / 4 ) == 2 &&
+            b2.top() > b1.bottom() && b2.hcenter() < b1.left() )
           {
-          int col, limit = b2.seek_right( b2.vcenter(), b2.hcenter() );
-          for( col = b2.hcenter(); col <= limit; ++col )
-            if( b2.seek_bottom( b2.vcenter(), col ) < b2.bottom() ) break;
-          if( b2.left() < col && col < b2.right() )
+          Character c2( b2 ); c2.recognize1( charset, vcenter() );
+          if( c2.maybe('n') )
             {
-            only_guess( 0, b2.left() );
-            if( charset.enabled( Charset::iso_8859_9 ) && f2.rp.istip() )
-              { add_guess( 'T', col - 1 ); add_guess( UCS::CIDOT, b2.right() ); }
-            else
+            int col, limit = b2.seek_right( b2.vcenter(), b2.hcenter() );
+            for( col = b2.hcenter(); col <= limit; ++col )
+              if( b2.seek_bottom( b2.vcenter(), col ) < b2.bottom() ) break;
+            if( b2.left() < col && col < b2.right() )
               {
-              add_guess( 'r', col - 1 );
-              if( code == '.' ) add_guess( 'i', b2.right() );
-              else add_guess( UCS::SIACUTE, b2.right() );
+              only_guess( 0, b2.left() );
+              if( charset.enabled( Charset::iso_8859_9 ) && f2.rp.istip() )
+                { add_guess( 'T', col - 1 ); add_guess( UCS::CIDOT, b2.right() ); }
+              else
+                {
+                add_guess( 'r', col - 1 );
+                if( code == '.' ) add_guess( 'i', b2.right() );
+                else add_guess( UCS::SIACUTE, b2.right() );
+                }
+              return;
               }
-            return;
             }
           }
-        }
 
-      if( code == '.' && f2.bp.minima( b2.height() / 4 ) == 1 &&
-          b1.bottom() <= b2.top() )
-        {
-        int hdiff;
-        if( b2.bottom_hook( &hdiff ) && std::abs( hdiff ) >= b2.height() / 2 )
+        if( code == '.' && f2.bp.minima( b2.height() / 4 ) == 1 &&
+            b1.bottom() <= b2.top() )
           {
-          if( hdiff > 0 && f2.rp.increasing( f2.rp.pos( 75 ) ) )
-            { add_guess( 'j', 0 ); return; }
-          if( hdiff < 0 )
+          int hdiff;
+          if( b2.bottom_hook( &hdiff ) && std::abs( hdiff ) >= b2.height() / 2 )
             {
-            if( charset.enabled( Charset::iso_8859_15 ) ||
-                charset.enabled( Charset::iso_8859_9 ) )
-              if( f2.wp.max() > 2 * f1.wp.max() && f2.lp.minima() == 1 )
-                { add_guess( UCS::IQUEST, 0 ); return; }
+            if( hdiff > 0 && f2.rp.increasing( f2.rp.pos( 75 ) ) )
+              { add_guess( 'j', 0 ); return; }
+            if( hdiff < 0 )
+              {
+              if( charset.enabled( Charset::iso_8859_15 ) ||
+                  charset.enabled( Charset::iso_8859_9 ) )
+                if( f2.wp.max() > 2 * f1.wp.max() && f2.lp.minima() == 1 )
+                  { add_guess( UCS::IQUEST, 0 ); return; }
+              add_guess( 'i', 0 ); return;
+              }
+            }
+          if( f2.tp.minima() == 1 )
+            {
+            if( Ocrad::similar( f1.wp.max(), f2.wp.max(), 20 ) )
+              {
+              if( charset.enabled( Charset::iso_8859_15 ) ||
+                  charset.enabled( Charset::iso_8859_9 ) )
+                if( f2.wp.max() >= f1.wp.max() &&
+                    3 * f2.wp[f2.wp.pos(10)] < 2 * f1.wp.max() )
+                  { add_guess( UCS::IEXCLAM, 0 ); return; }
+              add_guess( 'i', 0 ); return;
+              }
+            if( 3 * f2.wp.max() > 4 * f1.wp.max() &&
+                b2.seek_bottom( b2.vcenter(), b2.hpos( 10 ) ) < b2.bottom() &&
+                f2.rp.increasing( f2.rp.pos( 75 ) ) )
+              { add_guess( 'j', 0 ); return; }
+            if( charset.enabled( Charset::iso_8859_9 ) && f2.rp.istip() )
+              { add_guess( UCS::CIDOT, 0 ); return; }
             add_guess( 'i', 0 ); return;
             }
           }
-        if( f2.tp.minima() == 1 )
-          {
-          if( Ocrad::similar( f1.wp.max(), f2.wp.max(), 20 ) )
-            {
-            if( charset.enabled( Charset::iso_8859_15 ) ||
-                charset.enabled( Charset::iso_8859_9 ) )
-              if( f2.wp.max() >= f1.wp.max() &&
-                  3 * f2.wp[f2.wp.pos(10)] < 2 * f1.wp.max() )
-                { add_guess( UCS::IEXCLAM, 0 ); return; }
-            add_guess( 'i', 0 ); return;
-            }
-          if( 3 * f2.wp.max() > 4 * f1.wp.max() &&
-              b2.seek_bottom( b2.vcenter(), b2.hpos( 10 ) ) < b2.bottom() &&
-              f2.rp.increasing( f2.rp.pos( 75 ) ) )
-            { add_guess( 'j', 0 ); return; }
-          if( charset.enabled( Charset::iso_8859_9 ) && f2.rp.istip() )
-            { add_guess( UCS::CIDOT, 0 ); return; }
-          add_guess( 'i', 0 ); return;
-          }
         }
       }
-      }
 
-    if( !b1.blocks() )		// upper block has no holes
+    // upper block has no holes
+    if( ( b1.bottom() < b2.vcenter() || 2 * a1 < a2 ) && !b1.blocks() )
       {
       Character c( b2 ); c.recognize1( charset, c.vcenter() );
       if( c.guesses() )
         {
-        int code = c.guess( 0 ).code, atype = '\'';
-        if( UCS::isvowel( code ) &&
-            b1.width() > 2 * b1.height() && f1.hp.iscpit() )
-          atype = ':';
-        else if( f1.bp.minima() == 2 ) atype = '^';
-        else
+        int code = c.guess( 0 ).code;
+        if( b1.bottom() < b2.vcenter() )
           {
-          int slope; if( f1.rp.straight( &slope ) && slope < 0 ) atype = '`';
-          }
-        code = UCS::compose( code, atype );
-        if( charset.only( Charset::ascii ) )
-          {
-          if( UCS::base_letter( code ) == 'i' ) code = 'i';
-          else code = c.guess( 0 ).code;
+          int atype = '\'';
+          if( UCS::isvowel( code ) &&
+              b1.width() > 2 * b1.height() && f1.hp.iscpit() )
+            atype = ':';
+          else if( f1.bp.minima() == 2 ) atype = '^';
+          else
+            {
+            int slope; if( f1.rp.straight( &slope ) && slope < 0 ) atype = '`';
+            }
+          code = UCS::compose( code, atype );
+          if( charset.only( Charset::ascii ) )
+            {
+            if( UCS::base_letter( code ) == 'i' ) code = 'i';
+            else code = c.guess( 0 ).code;
+            }
           }
         if( code ) add_guess( code, 0 );
         }
