@@ -38,23 +38,36 @@ public:
   };
 
 
+class Transformation
+  {
+public:
+  enum Type { none, rotate90, rotate180, rotate270,
+              mirror_lr, mirror_tb, mirror_d1, mirror_d2 };
+private:
+  Type _type;
+
+public:
+  Transformation() throw() : _type( none ) {}
+  bool set( const char * name ) throw();
+  Type type() const throw() { return _type; }
+  void show_error( const char * arg ) const throw();
+  };
+
+
 struct Control
   {
   enum Format { byte, utf8 };
 
-  FILE *outfile, *exportfile;
+  Charset charset;
   Format format;
-  int rindex, debug_level, layout_level, scale, threshold;
-  char filetype, specialtype;
-  bool append, copy, force, invert, verbose;
+  FILE *outfile, *exportfile;
+  int debug_level;
+  bool verbose;
 
   Control() throw()
-    : outfile( stdout ), exportfile( 0 ),
-      format( byte ),
-      rindex( -1 ), debug_level( 0 ), layout_level( 0 ),
-      scale( 0 ), threshold( -1 ),
-      filetype( 0 ), specialtype( 0 ),
-      append( false ), copy( false ), force( false ), invert( false ),
+    : format( byte ),
+      outfile( stdout ), exportfile( 0 ),
+      debug_level( 0 ),
       verbose( false )
     {}
   bool set_format( const char * name ) throw();
