@@ -831,7 +831,8 @@ int Features::test_HKMNUuvwYy() const throw()
                   9 * wp[wp.pos(50)] > 10 * wp[wp.pos(70)] ) return 'v';
             else return 'u'; }
           }
-        if( hdiff > b.height() / 2 ) return 'y';
+        if( hdiff > b.height() / 2 )
+          { if( bp.minima( b.height() / 2 ) == 1 ) return 'y'; else return 0; }
         }
       int rg = rp[rp.samples()-2];
       if( bp.ispit() && 7 * tp.range() < 4 * b.height() &&
