@@ -26,32 +26,42 @@
 #include <cstdio>
 #include <cstdlib>
 #include <cstring>
+#include <string>
 #include <vector>
 #include <getopt.h>
 #include "common.h"
 #include "rectangle.h"
-#include "vrhomboid.h"
-#include "track.h"
 #include "bitmap.h"
-#include "block.h"
-#include "blockmap.h"
-#include "character.h"
-#include "textline.h"
-#include "textblock.h"
+#include "textpage.h"
 
 
 namespace {
 
-// Date of this version: 2004-05-23
+// Date of this version: 2004-10-23
 
-const char *const program_version = "0.8";
-const char *const program_year    = "2004";
+const char * const program_version = "0.9";
+const char * const program_year    = "2004";
+
+struct Input_control
+  {
+  Transformation transformation;
+  int rindex, layout_level, scale, threshold;
+  char filetype, specialtype;
+  bool copy, invert;
+
+  Input_control() throw()
+    : rindex( -1 ), layout_level( 0 ), scale( 0 ), threshold( -1 ),
+      filetype( 0 ), specialtype( 0 ),
+      copy( false ), invert( false )
+    {}
+  bool set_format( const char * name ) throw();
+  };
 
 
 void show_version() throw()
   {
   std::printf( "GNU Ocrad version %s\n", program_version );
-  std::printf( "Copyright (C) %s Antonio Diaz.\n", program_year );
+  std::printf( "Copyright (C) %s Antonio Diaz Diaz.\n", program_year );
   std::printf( "This program is free software; you may redistribute it under the terms of\n" );
   std::printf( "the GNU General Public License.  This program has absolutely no warranty.\n" );
   }
@@ -59,9 +69,9 @@ void show_version() throw()
 
 void show_error( const char * program_name, const char * msg ) throw()
   {
-  if( msg && msg[0] != 0 ) std::fprintf( stderr,"ocrad: %s\n", msg );
+  if( msg && msg[0] != 0 ) std::fprintf( stderr, "ocrad: %s\n", msg );
   if( program_name && program_name[0] != 0 )
-    std::fprintf( stderr,"Try `%s --help' for more information.\n", program_name );
+    std::fprintf( stderr, "Try `%s --help' for more information.\n", program_name );
   }
 
 
@@ -81,6 +91,7 @@ void show_help( const char * program_name, bool verbose ) throw()
   std::printf( "  -i, --invert             invert image levels (white on black)\n");
   std::printf( "  -l, --layout=<n>         layout analysis, 0=none, 1=column, 2=full\n");
   std::printf( "  -o <file>                place the output into <file>\n");
+  std::printf( "  -t, --transform=<name>   try `--transform=help' for a list of names\n");
   std::printf( "  -v, --verbose            be verbose\n");
   std::printf( "  -x <file>                export OCR Results File to <file>\n");
   if( verbose )
@@ -96,126 +107,94 @@ void show_help( const char * program_name, bool verbose ) throw()
   }
 
 
-void ocr( const Bitmap & page_image, const Charset & charset,
-                                     const Control & control, int c ) throw()
+const char * my_basename( const char * filename ) throw()
   {
-  Blockmap blockmap( page_image, c, control.debug_level );
-  if( !blockmap.blocks() ) return;
-
-  if( control.debug_level >= 90 )
-    {
-    if( control.outfile )
-      blockmap.print( control.outfile, control.debug_level );
-    return;
-    }
-
-  Textblock textblock( charset, blockmap.block_matrix() );
-
-  if( control.debug_level >= 86 )
-    {
-    if( control.outfile )
-      {
-      bool graph = ( control.debug_level >= 88 );
-      bool recursive = ( control.debug_level & 1 );
-      textblock.dprint( control, graph, recursive );
-      }
-    return;
-    }
-
-  if( control.debug_level == 0 )
-    {
-    if( control.outfile ) textblock.print( control );
-    if( control.exportfile ) textblock.xprint( control );
-    }
+  const char * c = filename;
+  while( *c ) { if( *c == '/' ) filename = c + 1; ++c; }
+  return filename;
   }
 
 
-int process_file( FILE *infile, const Charset & charset,
-                                const Control & control ) throw()
+int process_file( FILE *infile, const char * infile_name,
+                  const Input_control & input_control,
+                  const Control & control ) throw()
   {
   try
     {
-    Bitmap page_image( infile, control.invert );
+    Bitmap page_image( infile, input_control.invert );
 
-    page_image.analyse_layout( control.layout_level );
+    page_image.transform( input_control.transformation );
+    page_image.analyse_layout( input_control.layout_level );
     if( control.verbose ) std::fprintf( stderr, "number of text blocks %d\n",
                                 page_image.rectangles() );
 
-    if( control.rindex >= page_image.rectangles() )
+    if( input_control.rindex >= 0 &&
+        !page_image.only_this_rectangle( input_control.rindex ) )
       {
       std::fprintf( stderr,"This page has only %d text block(s)\n",
                     page_image.rectangles() );
       return 1;
       }
 
-    if( control.scale != 0 )
+    if( input_control.scale != 0 )
       {
       if( control.outfile )
         {
-        Bitmap reduced( page_image, control.scale, control.threshold );
-        reduced.save( control.outfile, control.filetype );
+        Bitmap reduced( page_image, input_control.scale, input_control.threshold );
+        reduced.save( control.outfile, input_control.filetype );
         }
       return 0;
       }
-    else if( control.specialtype != 0 )
+    else if( input_control.specialtype != 0 )
       {
       if( control.outfile )
         {
-        Bitmap::type t;
-        if( control.specialtype == 'v' ) t = Bitmap::vertical_histogram ;
-        else if( control.specialtype == 'h' ) t = Bitmap::horizontal_histogram ;
-        else if( control.specialtype == 'g' ) t = Bitmap::connected_ground ;
+        Bitmap::Type t;
+        if( input_control.specialtype == 'v' ) t = Bitmap::vertical_histogram;
+        else if( input_control.specialtype == 'h' ) t = Bitmap::horizontal_histogram;
+        else if( input_control.specialtype == 'g' ) t = Bitmap::connected_ground;
         else { show_error( 0, "bad special type" ); return 1; }
         Bitmap sb( page_image, t );
-        sb.save( control.outfile, control.filetype );
+        sb.save( control.outfile, input_control.filetype );
         }
       return 0;
       }
-    else if( control.copy )
+    else if( input_control.copy )
       {
       if( control.outfile )
         {
-        if( control.rindex < 0 && page_image.rectangles() == 1 )
-          { page_image.save( control.outfile, control.filetype ); return 0; }
+        if( input_control.layout_level == 0 && page_image.rectangles() == 1 )
+          { page_image.save( control.outfile, input_control.filetype ); return 0; }
         for( int c = 0; c < page_image.rectangles(); ++c )
-          if( control.rindex < 0 || control.rindex == c )
-            {
-            Bitmap bitmap( page_image, page_image.rectangle_vector()[c] );
-            bitmap.save( control.outfile, control.filetype );
-            }
+          {
+          Bitmap bitmap( page_image, page_image.rectangle_vector()[c] );
+          bitmap.save( control.outfile, input_control.filetype );
+          }
         }
       return 0;
       }
 
-    if( control.exportfile )
-      std::fprintf( control.exportfile, "total blocks %d\n",
-                    ( control.rindex < 0 ) ? page_image.rectangles() : 1 );
+    Textpage textpage( page_image, my_basename( infile_name ), control );
 
-    // call the character recognizer for every rectangle of text
-    for( int c = 0; c < page_image.rectangles(); ++c )
-      if( control.rindex < 0 || control.rindex == c )
+    if( control.debug_level >= 86 && control.debug_level <= 89 )
+      {
+      if( control.outfile )
         {
-        if( control.exportfile )
-          {
-          const Rectangle & r = page_image.rectangle_vector()[c];
-          std::fprintf( control.exportfile, "block %d %d %d %d %d\n", c + 1,
-                        r.left(), r.top(), r.width(), r.height() );
-          }
-        ocr( page_image, charset, control, c );
+        bool graph = ( control.debug_level >= 88 );
+        bool recursive = ( control.debug_level & 1 );
+        textpage.dprint( control, graph, recursive );
         }
+      }
+    else if( control.debug_level == 0 )
+      {
+      if( control.outfile ) textpage.print( control );
+      if( control.exportfile ) textpage.xprint( control );
+      }
     }
   catch( Bitmap::Error e ) { std::fputs( e.s, stderr ); return 2; }
   return 0;
   }
 
-
-const char * my_basename( const char * filename ) throw()
-  {
-  const char * c = filename;
-  while( *c ) { if( *c == '/' ) filename = c + 1; ++c; }
-  return filename;
-  }
-
 } // end namespace
 
 
@@ -226,15 +205,16 @@ const char * my_basename( const char * filename ) throw()
 // 'exportfile' is the Ocr Results File.
 int main( int argc, char *argv[] ) throw()
   {
-  Charset charset;
+  Input_control input_control;
   Control control;
   char *outfile_name = 0, *exportfile_name = 0;
+  bool append = false, force = false;
 
   // scan arguments
 
   while( true )			// process options
     {
-    static struct option long_options[] =
+    static const struct option long_options[] =
       {
       {"append", no_argument, 0, 'a'},
       {"block", required_argument, 0, 'b'},
@@ -246,12 +226,13 @@ int main( int argc, char *argv[] ) throw()
       {"help", no_argument, 0, 'h'},
       {"invert", no_argument, 0, 'i'},
       {"layout", required_argument, 0, 'l'},
+      {"transform", required_argument, 0, 't'},
       {"verbose", no_argument, 0, 'v'},
       {"version", no_argument, 0, 'V'},
       {0, 0, 0, 0}
       };
 
-    int c = getopt_long( argc, argv, "14ab:c:CD:fF:hil:o:R:S:T:Vvx:",
+    int c = getopt_long( argc, argv, "14CD:F:R:S:T:Vab:c:fhil:o:t:vx:",
                          long_options, 0 );
     if( c == -1 ) break;		// all options processed
 
@@ -260,27 +241,30 @@ int main( int argc, char *argv[] ) throw()
       case 0: break;
       case '?': show_error( argv[0], 0 ); return 1;	// bad option
       case '1':
-      case '4': control.filetype = c; break;
-      case 'a': control.append = true; break;
-      case 'b': control.rindex = std::strtol( optarg, 0, 0 ) - 1; break;
-      case 'c': if( !charset.enable( optarg ) )
-                  { charset.show_error( optarg ); return 1; }
-                break;
-      case 'C': control.copy = true; break;
+      case '4': input_control.filetype = c; break;
+      case 'C': input_control.copy = true; break;
       case 'D': control.debug_level = std::strtol( optarg, 0, 0 ); break;
-      case 'f': control.force = true; break;
       case 'F': if( !control.set_format( optarg ) )
                   { show_error( argv[0], "bad output format" ); return 1; }
                 break;
-      case 'h': show_help( argv[0], control.verbose ); return 0;
-      case 'i': control.invert = true; break;
-      case 'l': control.layout_level = std::strtol( optarg, 0, 0 ); break;
-      case 'o':	outfile_name = optarg; break;
-      case 'R': control.scale = std::strtol( optarg, 0, 0 ); break;
-      case 'S': control.specialtype = optarg[0]; break;
-      case 'T': control.threshold = std::strtol( optarg, 0, 0 ); break;
-      case 'v': control.verbose = true; break;
+      case 'R': input_control.scale = std::strtol( optarg, 0, 0 ); break;
+      case 'S': input_control.specialtype = optarg[0]; break;
+      case 'T': input_control.threshold = std::strtol( optarg, 0, 0 ); break;
       case 'V':	show_version(); return 0;
+      case 'a': append = true; break;
+      case 'b': input_control.rindex = std::strtol( optarg, 0, 0 ) - 1; break;
+      case 'c': if( !control.charset.enable( optarg ) )
+                  { control.charset.show_error( optarg ); return 1; }
+                break;
+      case 'f': force = true; break;
+      case 'h': show_help( argv[0], control.verbose ); return 0;
+      case 'i': input_control.invert = true; break;
+      case 'l': input_control.layout_level = std::strtol( optarg, 0, 0 ); break;
+      case 'o':	outfile_name = optarg; break;
+      case 't': if( !input_control.transformation.set( optarg ) )
+                  { input_control.transformation.show_error( optarg ); return 1; }
+                break;
+      case 'v': control.verbose = true; break;
       case 'x':	exportfile_name = optarg; break;
       default: return 1;
       }
@@ -288,8 +272,8 @@ int main( int argc, char *argv[] ) throw()
 
   if( outfile_name && std::strcmp( outfile_name, "-" ) != 0 )
     {
-    if( control.append ) control.outfile = std::fopen( outfile_name, "a" );
-    else if( control.force ) control.outfile = std::fopen( outfile_name, "w" );
+    if( append ) control.outfile = std::fopen( outfile_name, "a" );
+    else if( force ) control.outfile = std::fopen( outfile_name, "w" );
     else if( ( control.outfile = std::fopen( outfile_name, "wx" ) ) == 0 )
       {
       std::fprintf( stderr, "Output file %s already exists.\n", outfile_name );
@@ -299,8 +283,9 @@ int main( int argc, char *argv[] ) throw()
       { std::fprintf( stderr, "Cannot open %s\n", outfile_name ); return 1; }
     }
 
-  if( exportfile_name && control.debug_level == 0 && control.scale == 0 &&
-      control.specialtype == 0 && !control.copy )
+  if( exportfile_name && control.debug_level == 0 &&
+      input_control.scale == 0 && input_control.specialtype == 0 &&
+      !input_control.copy )
     {
     if( std::strcmp( exportfile_name, "-" ) == 0 )
       { control.exportfile = stdout; if( !outfile_name ) control.outfile = 0; }
@@ -342,11 +327,7 @@ int main( int argc, char *argv[] ) throw()
       }
     if( !infile ) break;
 
-    if( control.exportfile )
-      std::fprintf( control.exportfile, "source file %s\n",
-                    my_basename( infile_name ) );
-
-    int tmp = process_file( infile, charset, control );
+    int tmp = process_file( infile, infile_name, input_control, control );
     if( tmp > 0 && infile == stdin ) infile = 0;
     if( tmp > retval ) retval = tmp;
     if( control.outfile ) std::fflush( control.outfile );
