@@ -25,8 +25,8 @@
 #include "profile.h"
 
 
-Profile::Profile( const Block & b, type t ) throw()
-  : _block( &b ), _blockmap( b.blockmap() ), _type(t), valid(false),
+Profile::Profile( const Block & b, Type t ) throw()
+  : _block( &b ), _blockmap( b.blockmap() ), type(t), valid(false),
     valid_max(false), valid_min(false), valid_mean(false),
     valid_range(false), valid_isconcave(false),
     valid_isconvex(false), valid_isflat(false), valid_isflats(false),
@@ -40,7 +40,7 @@ void Profile::initialize() throw()
   const Blockmap & bm = *_blockmap;
 
   valid = true;
-  switch( _type )
+  switch( type )
     {
     case left :
       data.resize( b.height() ); _limit = b.width();
