@@ -19,12 +19,12 @@
 class Profile
   {
 public:
-  enum type { left, top, right, bottom, height, width };
+  enum Type { left, top, right, bottom, height, width };
 
 private:
   const Block * _block;		// Block to witch this profile belongs
   const Blockmap * _blockmap;	// Blockmap to which the block belongs
-  type _type;
+  Type type;
   bool valid, valid_max, valid_min, valid_mean, valid_range;
   bool valid_isconcave, valid_isconvex;
   bool valid_isflat, valid_isflats, valid_ispit, valid_isupit;
@@ -37,7 +37,7 @@ private:
   int mean() throw();
 
 public:
-  Profile( const Block & b, type t ) throw();
+  Profile( const Block & b, Type t ) throw();
 
   const Block * block() const throw() { return _block; }
   const Blockmap * blockmap() const throw() { return _blockmap; }
