@@ -39,8 +39,10 @@ public:
 
   int hcenter() const throw() { return ( _left + _right ) / 2; }
   int vcenter() const throw() { return ( _top + _bottom ) / 2; }
-  int hpos( int p ) const throw();
-  int vpos( int p ) const throw();
+  int hpos( int p ) const throw()
+    { return _left + ( ( ( _right - _left ) * p ) / 100 ); }
+  int vpos( int p ) const throw()
+    { return _top + ( ( ( _bottom - _top ) * p ) / 100 ); }
 
   void add_point( int row, int col ) throw();
   void add_rectangle( const Rectangle & r ) throw();
@@ -56,6 +58,8 @@ public:
   bool v_includes( int row ) const throw();
   bool h_overlaps( const Rectangle & r ) const throw();
   bool v_overlaps( const Rectangle & r ) const throw();
+  bool h_precedes( const Rectangle & r ) const throw();
+  bool v_precedes( const Rectangle & r ) const throw();
 
   int distance( const Rectangle & r ) const throw();
   int distance( int row, int col ) const throw();
