@@ -59,8 +59,9 @@ void Textblock::join_characters() throw()
   }
 
 
-Textblock::Textblock( const Charset & charset,
+Textblock::Textblock( const Rectangle & r, const Charset & charset,
            const std::vector< std::vector< Block > > & block_matrix ) throw()
+  : Rectangle( r )
   {
   int cuts = block_matrix.size();
   int mean_height[cuts];
