@@ -16,7 +16,7 @@
     Foundation, 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
 
-class Textblock
+class Textblock : public Rectangle
   {
   mutable std::vector< Textline > data;
 
@@ -26,7 +26,7 @@ class Textblock
   void join_characters() throw();
 
 public:
-  Textblock( const Charset & charset,
+  Textblock( const Rectangle & r, const Charset & charset,
              const std::vector< std::vector< Block > > & block_matrix ) throw();
 
   Textline & line( int i ) const throw();
